@@ -3,7 +3,7 @@
 #include "frameobject.h"
 #include "pythread.h"
 
-#define VERSION "0.8.1"
+#define VERSION "0.9"
 
 #if PY_MAJOR_VERSION >= 3
 # define PYTHON3
@@ -292,10 +292,10 @@ trace_free(trace_free_t func, void *ptr)
 static void
 trace_free_list_alloc(PyObject *op)
 {
-    void *ptr;
-    PyTypeObject *type;
-    size_t size;
     trace_alloc_t *trace;
+    PyTypeObject *type;
+    void *ptr;
+    size_t size;
 
     if (!trace_config.enabled)
         return;
@@ -318,13 +318,18 @@ trace_free_list_alloc(PyObject *op)
 static void
 trace_free_list_free(PyObject *op)
 {
-    trace_alloc_t *trace;
+    PyTypeObject *type;
     void *ptr;
+    trace_alloc_t *trace;
 
     if (!trace_config.enabled)
         return;
 
-    ptr = (void *)op;
+    type = Py_TYPE(op);
+    if (PyType_IS_GC(type))
+        ptr = (void *)((char *)op - sizeof(PyGC_Head));
+    else
+        ptr = (void *)op;
     trace = g_hash_table_lookup(trace_allocs, ptr);
     if (trace != NULL)
         trace_log_dealloc(ptr, trace);
@@ -803,13 +808,13 @@ static PyObject*
 py_trace_get_stats(PyObject *self)
 {
     int was_enabled;
+    trace_get_stats_t get_stats;
 
     /* don't track memory allocations done by trace_get_stats_fill_file()
      * to not modify hash tables */
     was_enabled = trace_config.enabled;
     trace_config.enabled = 0;
 
-    trace_get_stats_t get_stats;
     get_stats.file_dict = PyDict_New();
     if (get_stats.file_dict == NULL)
         goto done;
